<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
class Check {
	public $spec;
	public $width;
	public $required;

	public function error ($input) {
		if (!is_string($input))
			return "Must be string";
		if ($this->width && (strlen($input)>$this->width))
			return "Too long. Limit " . $this->width . " characters.  You have ". strlen($input) . ".";
		if ($this->required && $input == '')
			return "Required. Must not be blank.";
		return false;
	}
}

class email_Check extends Check {
	public function error ($input) {
		if ($error = parent::error($input))
			return $error;
		if (!$input)
			return false;
		if (!strpos($input, "@"))
			return "Expected @, as in john@example.com";
		if (preg_match ('/([^A-Za-z0-9\!\#\$\%\&\*\'\+\-\/\=\?\^\_\`\{\|\}\~\.\@])/', $input, $result))
			return "Unexpected '" . htmlentities($result[1]) . "'";
	}
}

class numeric_Check extends Check {
	public function error ($input) {
		if ($error = parent::error($input))
			return $error;
		if (!$input)
			return false;
		if (!preg_match ('/^\d*$/', $input))
			return "Must be numeric.";
	}
}

class timestamp_Check extends Check {
	public function error ($input) {
		if ($error = parent::error($input))
			return $error;
		if (!$input)
			return false;
		if (!preg_match ('/^\d{4}-\d{2}-\d{2}[ ]\d{2}:\d{2}:\d{2}$/', $input))
			return "Timestamp format is YYYY-MM-DD hh:mm:ss.";
	}
}

class date_Check extends Check {
	public function error ($input) {
	//	if ($error = parent::error($input))
	//		return $error;
		if ($input == null)
			return false;
	return strtotime($input);	
		}
		
	public static function standardDate($input, $type = 'short') { 
		// $type can be mysqlDate1, long, short, or null
		$obj = new date_Check;
		$myDate = $obj->error($input);
//		var_dump($myDate);
		if ($myDate) {
			$dateArray = getDate($myDate);
			switch ($type) {
			case 'mysqlDate1' : // YYYY-mm-dd
				$myYear = $dateArray['year'];
				$myMonth = $dateArray['mon'];
				$myDoM = $dateArray['mday'];
				$formattedDate =  $myYear . '-' . $myMonth . '-' . $myDoM ;
				return $formattedDate;
				break;
			case 'long' : 
				$myDay = $dateArray['weekday'];
				$myMonth = $dateArray['month'];
				$myDoM = $dateArray['mday'];
				$myYear = $dateArray['year'];
				$formattedDate = $myDay . ', ' . $myMonth . ' ' . $myDoM . ', ' . $myYear;
				return $formattedDate;
				break;
			case 'short' :
			case '' :
				$myMonth = $dateArray['mon'];
				$myDoM = $dateArray['mday'];
				$myYear = substr($dateArray['year'],2);
				$formattedDate = $myMonth . '/' . $myDoM . '/' . $myYear;
				return $formattedDate;
				break;
			default :
				return 'Unrecognized type: ' . $type;
				}
		}
		else
			return 'Bad date format, use something more reasonable: ' . $input;
	}
}


class vote_Check extends Check {
	public function error ($input) {
		if ($error = parent::error($input))
			return $error;
		if (!$input && $input != '0')
			return false;
		if( $input == '+1'
		 || $input == '0'
		 || $input == '-1')
		 	return false;
		return "Must be +1, 0, or -1";
	}
}

class yesno_Check extends Check {
	public function error ($input) {
		if ($error = parent::error($input))
			return $error;
		if (!$input)
			return false;
		if( $input == 'yes'
		 || $input == 'no')
		 	return false;
		return "Must be yes or no.";
	}
}
class noquery_Check extends Check {
	public function error( $input ) {
		if( $error = parent::error($input)) 
			return $error;
		if( !$input )
			return false;
		if( preg_match("/bugs.eclipse.org\/bugs\/buglist.cgi?/", $input))
			return "Must use actual bug numbers [NNN], not queries.";
		if( preg_match("/www.eclipse.org\/search\/search.cgi/", $input))
			return "Must use concrete urls, not searches.";
	}
}

// sample check specs:
//	email
//	email 30
//	req email
//	req email 30
//	req 30
//	req
//	30

function check_factory($spec) {
	if (!preg_match('/(req)?\s*([a-z]+)?\s*(\d*)$/', $spec, $result))
		throw new Exception("can't parse check spec");
	$name = "Check";
	if (isset($result[2]) && $result[2])
		$name = $result[2] . "_Check";
	$check = new $name ();
	$check->spec = $spec;
	if (isset($result[3]))
		$check->width=intval($result[3]);
	if (isset($result[1]))
		$check->required=$result[1];
	return $check;
}

